﻿using System;
using System.Web.Mvc;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using Telerik.Web.UI.Common;
using System.Text;
using System.Text.RegularExpressions;

namespace Telerik.Web.Mvc
{
    public static class RadControlsHtmlHelpers
    {
        private static int idCounter = 0;
        public static string RadControlCss<T>(this HtmlHelper htmlHelper, Action<T> initializeControl)
            where T : WebControl, ISkinnableControl, new()
        {
            var control = CreateControl(initializeControl);
            control.EnableEmbeddedBaseStylesheet = true;
            control.EnableEmbeddedSkins = true;

            return ControlRenderer.GetControlCssReferences(control);
        }

        public static string RadControl<T>(this HtmlHelper htmlHelper, Action<T> initializeControl)
            where T : WebControl, ISkinnableControl, new()
        {
            return ControlRenderer.GetControlHtml(CreateControl(initializeControl));
        }

        private static T CreateControl<T>(Action<T> initializeControl) where T : WebControl, ISkinnableControl, new()
        {
            var id = "RadControl" + (++idCounter).ToString();
            var control = new T
            {
                ID = id,
                RegisterWithScriptManager = false,
                EnableEmbeddedSkins = false,
                EnableEmbeddedBaseStylesheet = false,
                EnableEmbeddedScripts = false
            };
            var page = new Page();
            page.Controls.Add(control);

            initializeControl(control);
            if (control is RadTimePicker)
            {
                var picker = (control as RadTimePicker);
                picker.TimeView.DataBind();
            }
            return control;
        }

        public static string RadControlScripts<T>(this HtmlHelper htmlHelper, Action<T> initializeControl)
            where T : WebControl, ISkinnableControl, new()
        {
            var control = CreateControl(initializeControl);
            control.EnableEmbeddedScripts = true;
            return ControlRenderer.GetControlScripts(control);
        }

        public static string RadControlScripts<T>(this HtmlHelper htmlHelper)
            where T : WebControl, ISkinnableControl, new()
        {
            return RadControlScripts<T>(htmlHelper, delegate { });
        }

        public static string RadControlCss<T>(this HtmlHelper htmlHelper) where T : WebControl, ISkinnableControl, new()
        {
            return RadControlCss<T>(htmlHelper, delegate { });
        }

        public static string RadControl<T>(this HtmlHelper htmlHelper)
            where T : WebControl, ISkinnableControl, new()
        {
            return RadControl<T>(htmlHelper, delegate { });
        }
    }

    public static class RadDatePickerExtensions
    {
        public static string RenderRadDatePicker(this HtmlHelper helper, Action<RadDatePicker> initializeControl, string skinName)
        {
            var output = new StringBuilder();
            output.AppendLine(helper.RadControlCss<RadCalendar>(control => control.Skin = skinName));
            output.AppendLine(helper.RadControlCss<RadDateInput>(control => control.Skin = skinName));
            output.AppendLine(helper.RadControlScripts<RadDatePicker>(control => control.Skin = skinName));
            output.AppendLine(helper.RadControlScripts<RadDateInput>(control => control.Skin = skinName));
            output.AppendLine(helper.RadControlScripts<RadCalendar>(control => control.Skin = skinName));
            Action<RadDatePicker> func = control =>
                                             {
                                                 control.Skin = skinName;
                                                 initializeControl(control);
                                             };
            output.AppendLine(helper.RadControl(func));
            return output.ToString();
        }

        public static string RenderRadTimePicker(this HtmlHelper helper, Action<RadTimePicker> initializeControl, string skinName)
        {
            var output = new StringBuilder();
            output.AppendLine(helper.RadControlCss<RadTimeView>(control => control.Skin = skinName));
            output.AppendLine(helper.RadControlCss<RadCalendar>(control => control.Skin = skinName));
            output.AppendLine(helper.RadControlCss<RadDateInput>(control => control.Skin = skinName));
            output.AppendLine(helper.RadControlScripts<RadTimePicker>(control => control.Skin = skinName));
            output.AppendLine(helper.RadControlScripts<RadCalendar>(control => control.Skin = skinName));
            output.AppendLine(helper.RadControlScripts<RadDateInput>(control => control.Skin = skinName));
            Action<RadTimePicker> func = control =>
            {
                control.Skin = control.TimeView.Skin = skinName;
                initializeControl(control);
                SetStyleClasses(control.TimeView);
            };
            output.AppendLine(helper.RadControl(func));
            return output.ToString();
        }

        private static void SetStyleClasses(RadTimeView view)
        {
            view.TimeStyle.CssClass = FormatCssClass(view, "", view.TimeStyle.CssClass);
            view.AlternatingTimeStyle.CssClass = FormatCssClass(view, "", view.AlternatingTimeStyle.CssClass);
            view.HeaderStyle.CssClass = FormatCssClass(view, "rcHeader", view.HeaderStyle.CssClass);
            view.FooterStyle.CssClass = FormatCssClass(view, "rcFooter", view.FooterStyle.CssClass);
            view.TimeOverStyle.CssClass = FormatCssClass(view, "rcHover", view.TimeOverStyle.CssClass);
            view.CssClass = FormatCssClass(view, "RadCalendarTimeView", view.CssClass);
        }

        private static string FormatCssClass(RadTimeView view, string prefix, string userDefined)
        {
            string skinCssClass = "";
            if (prefix == "RadCalendarTimeView")
            {
                skinCssClass = String.IsNullOrEmpty(view.Skin) ? prefix : String.Format("{0} {0}_{1}", prefix, view.Skin);
            }
            else
            {
                skinCssClass = prefix;
            }
            userDefined = Regex.Replace(userDefined, prefix + "_\\S+\\s?", "");

            if (userDefined.IndexOf(skinCssClass) >= 0)
                return userDefined;
            else
                if (userDefined == String.Empty)
                    return skinCssClass;
                else
                    return String.Format("{0} {1}", skinCssClass, userDefined);
        }
    }

    public static class RadTextBoxExtensions 
    {
        public static string RenderRadTextBox(this HtmlHelper helper, Action<RadTextBox> initializeControl, string skinName)
        {
            var output = new StringBuilder();
            output.AppendLine(helper.RadControlCss<RadTextBox>(control => control.Skin = skinName));
            output.AppendLine(helper.RadControlScripts<RadTextBox>(control => control.Skin = skinName));
            Action<RadTextBox> func = control =>
            {
                control.Skin = skinName;
                initializeControl(control);
            };
            output.AppendLine(helper.RadControl(func));
            return output.ToString();
        }
    }

    public static class RadGridExtensions 
    {
        public static string RenderRadGrid(this HtmlHelper helper, Action<RadGrid> initializeControl, string skinName) 
        {
            var output = new StringBuilder();
            output.AppendLine(helper.RadControlCss<RadGrid>(control => control.Skin = skinName));
            output.AppendLine(helper.RadControlScripts<RadGrid>(control => control.Skin = skinName));
            Action<RadGrid> func = control =>
                                             {
                                                 control.Skin = skinName;
                                                 initializeControl(control);
                                             };
            output.AppendLine(helper.RadControl(func));
            return output.ToString();
        }
    }
}